from typing import List, Optional
from email_manager.providers.base import BaseEmailProvider


class EmailNotificationService:
    """
    Servicio de envío de correos usando un proveedor específico.
    """

    def __init__(self, provider: BaseEmailProvider):
        self.provider = provider
        self.from_email = None
        self.from_name = None

    def send_email(self, to: str, subject: str, body: str,  attachments: Optional[List[str]] = None, **kwargs) -> bool:
        """
        Envía un correo a un destinatario usando el proveedor configurado.
        """
        return self.provider.send_email(to, subject, body, attachments, **kwargs)

    def send_bulk_email(self, recipients: list, subject: str, body: str, attachments: Optional[List[str]] = None) -> dict:
        """
        Envía correos a múltiples destinatarios.
        """
        return self.provider.send_bulk_email(recipients, subject, body, attachments)

    def validate_email(self, email: str) -> bool:
        """
        Validación básica de formato de correo.
        """
        return "@" in email and "." in email


import requests
from email_manager.providers.base import BaseEmailProvider
from email_manager.providers.exceptions import EmailSendError

SEND_MESSAGE_MAILGUN = "https://api.mailgun.net/v3/{domain}"


class MailgunProvider(BaseEmailProvider):
    def __init__(self, mailgun_params: dict, config):
        
        self.api_key = mailgun_params.get("api_key")
        self.domain = mailgun_params.get("domain")
        self.base_url = SEND_MESSAGE_MAILGUN.format(domain=self.domain)
        self.from_email = mailgun_params.get("from_email")
        self.timeout = config.timeout
        self.max_retries = config.max_retries

    def send_email(self, to: str, subject: str, body: str, reply_to: str, bcc: list =None, attachments: list = None, **kwargs) -> bool:
        payload = self._build_mailgun_payload(to, subject, body, reply_to, bcc, **kwargs)
        try:
            response = self._make_request("/messages", payload, attachments)
            if not self._validate_response(response):
                raise EmailSendError(self.get_provider_name(), f"Respuesta inválida: {response.status_code}")
            return True
        except Exception as e:
            raise EmailSendError(self.get_provider_name(), str(e))


    def send_bulk_email(self, recipients: list, subject: str, body: str) -> dict:
        results = {}
        for recipient in recipients:
            results[recipient] = self.send_email(recipient, subject, body)
        return results

    def _build_mailgun_payload(self, to: str, subject: str, body: str, reply_to: str, bcc: list = None, **kwargs) -> dict:
        return {
            "from": self.from_email,
            "to": to,
            "subject": subject,
            "html": body,
            "h:Reply-To":reply_to,
            "bcc":bcc,
        }

    def _make_request(self, endpoint: str, data: dict, attachments: list):
        files = []
        file_handles = []
        try:
            if attachments:
                for attachment in attachments:
                    f = open(attachment, 'rb')
                    file_handles.append(f)
                    files.append(('attachment', f))
            response = requests.post(
                f"{self.base_url}{endpoint}",
                auth=("api", self.api_key),
                data=data,
                files=files if files else None,
                timeout=self.timeout
            )
            return response
        finally:
            for f in file_handles:
                f.close()


    def _validate_response(self, response) -> bool:
        return response.status_code == 200

    def _handle_error(self, error: Exception):
        print(f"[Mailgun Error] {error}")

    def get_provider_name(self) -> str:
        return "MailgunProvider"



from email_manager.config.email_config import EmailConfig
from email_manager.services.notification_manager import EmailNotificationManager

class EmailSystem:
    def __init__(
        self,
        provider_type: str,
        timeout: int = 10,
        max_retries: int = 3,
        mailgun_params: dict = None,
        smtp_params: dict = None,
        mailtrap_params: dict = None,
    ):
        config = EmailConfig(
            provider_type=provider_type,
            timeout=timeout,
            max_retries=max_retries,
            mailgun_params=mailgun_params,
            smtp_params=smtp_params,
            mailtrap_params=mailtrap_params,
        )
        self.manager = EmailNotificationManager(config)

    def send(self, to: str, subject: str, body: str, reply_to: str, bcc: list, attachments: list = None) -> bool:
        return self.manager.send_notification(to, subject, body, reply_to, bcc, attachments)



# email_system/factory/provider_factory.py

from email_manager.providers.mailgun_provider import MailgunProvider
from email_manager.providers.smtp_provider import SMTPProvider
from email_manager.providers.mailtrap_provider import MailtrapProvider
from email_manager.providers.base import BaseEmailProvider
from email_manager.config.email_config import EmailConfig



class ProviderFactory:
    _provider_map = {
        "mailgun": lambda config: MailgunProvider(mailgun_params=config.mailgun_params, config=config),
        "smtp": lambda config: SMTPProvider(smtp_params=config.smtp_params, config=config),
        "mailtrap": lambda config: MailtrapProvider(mailtrap_params=config.mailtrap_params, config=config)
    }

    @staticmethod
    def create_provider(config: EmailConfig, override: str = None) -> BaseEmailProvider:
        provider_key = (override or config.provider_type).lower()
        try:
            return ProviderFactory._provider_map[provider_key](config)
        except KeyError:
            raise ValueError(f"Unsupported provider type: {provider_key}")
        except Exception as e:
            raise RuntimeError(f"Error creating provider '{provider_key}': {e}")




# email_manager/providers/base.py
from abc import ABC, abstractmethod
from typing import List, Optional

class BaseEmailProvider(ABC):
    @abstractmethod
    def send_email(
        self,
        to: str,
        subject: str,
        body: str,
        bcc: Optional[List[str]] = None,
        attachments: Optional[List[str]] = None,
        **kwargs
    ) -> bool:
        """
        Envía un correo electrónico a un destinatario.
        """
        pass

    @abstractmethod
    def send_bulk_email(
        self,
        recipients: List[str],
        subject: str,
        body: str,
        bcc: Optional[List[str]] = None,
        attachments: Optional[List[str]] = None,
        **kwargs
    ) -> dict:
        """
        Envía correos electrónicos a múltiples destinatarios.
        """
        pass

    @abstractmethod
    def get_provider_name(self) -> str:
        """
        Devuelve el nombre del proveedor (por ejemplo, 'SMTPProvider').
        """
        pass

    def _handle_error(self, error: Exception):
        """
        Manejo genérico de errores.
        """
        print(f"[{self.get_provider_name()} Error] {error}")



from .mailgun_provider import MailgunProvider
from .smtp_provider import SMTPProvider
from .mailtrap_provider import MailtrapProvider

__all__ = ["MailgunProvider", "SMTPProvider", "MailtrapProvider"]



from email import encoders
from email.mime.base import MIMEBase
import smtplib
import ssl
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email_manager.providers.base import BaseEmailProvider
import os

class MailtrapProvider(BaseEmailProvider):
    def __init__(self, mailtrap_params: dict, config):
        self.host = mailtrap_params["host"]
        self.port = mailtrap_params["port"]
        self.password = mailtrap_params["psw"]
        self.from_email = mailtrap_params["from_email"]
        self.timeout = config.timeout

    def send_email(self, to: str, subject: str, body: str, reply_to: str = None, bcc: list = None, attachments: list = None, **kwargs) -> bool:
        try:
            to = [i.strip() for i in to.split(',')] if isinstance(to, str) else to
            bcc = bcc or []
            attachments = attachments or []

            msg = MIMEMultipart()
            msg["From"] = self.from_email
            msg["To"] = ", ".join(to)
            msg["Subject"] = subject
            msg['reply-to'] = reply_to
            msg['Bcc'] = ", ".join(bcc)

            msg.attach(MIMEText(body, "html"))

            for attachment in attachments:
                if not os.path.isfile(attachment):
                    raise FileNotFoundError(f"Archivo no encontrado: {attachment}")
                part = MIMEBase('application', 'octet-stream')
                with open(attachment, 'rb') as f:
                    part.set_payload(f.read())
                encoders.encode_base64(part)
                part.add_header('Content-Disposition', 'attachment', filename=os.path.basename(attachment))
                msg.attach(part)

            context = ssl.create_default_context()
            with smtplib.SMTP(self.host, self.port, timeout=self.timeout) as server:
                server.starttls(context=context)
                server.login(self.from_email, self.password)
                recipients = to + bcc
                server.sendmail(self.from_email, recipients, msg.as_string())

            return True
        except Exception as e:
            self._handle_error(e)
            return False


    def send_bulk_email(self, recipients: list, subject: str, body: str) -> dict:
        return {recipient: self.send_email(recipient, subject, body) for recipient in recipients}

    def _handle_error(self, error: Exception):
        print(f"[Mailtrap Error] {error}")

    def get_provider_name(self) -> str:
        return "MailtrapProvider"


from email.mime.base import MIMEBase
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email_manager.providers.base import BaseEmailProvider
from email import encoders


class SMTPProvider(BaseEmailProvider):
    def __init__(self, smtp_params: dict, config):
        self.host = smtp_params.get("host")
        self.port = smtp_params.get("port")
        self.from_email = smtp_params.get("from_email")
        self.password = smtp_params.get("psw")
        self.use_tls = smtp_params.get("use_tls")
        self.timeout = config.timeout

    def send_email(self, to: str, subject: str, body: str, reply_to:str, bcc: list = None, attachments: list = None, **kwargs) -> bool:
        try:
            message, recipients = self._create_message(to, subject, body, reply_to, bcc, attachments)
            smtp = self._connect_smtp()
            smtp.send_message(message, to_addrs=recipients)
            smtp.quit()
            print('se envio con smtp')
            return True
        except Exception as e:
            self._handle_error(e)
            return False



    def send_bulk_email(self, recipients: list, subject: str, body: str, bcc: list = None, attachments: list = None) -> dict:
        return {
            recipient: self.send_email(recipient, subject, body, bcc=bcc, attachments=attachments)
            for recipient in recipients
        }


    def _create_message(self, to: str, subject: str, body: str, reply_to: str = None, bcc: list = None, attachments: list = None) -> tuple:
        bcc = bcc or []
        attachments = attachments or []

        msg = MIMEMultipart()
        msg["From"] = self.from_email
        msg["To"] = to
        msg["Subject"] = subject
        msg['reply-to'] = reply_to
        msg['Bcc'] = ", ".join(bcc)
        
        msg.attach(MIMEText(body, "html"))

        for attachment in attachments:
            part = MIMEBase('application', 'octet-stream')
            with open(attachment, 'rb') as f:
                part.set_payload(f.read())
            encoders.encode_base64(part)
            part.add_header('Content-Disposition', 'attachment', filename=attachment.split('/')[-1])
            msg.attach(part)

        recipients = [to] if isinstance(to, str) else to
        recipients += bcc
        return msg, recipients


    def _connect_smtp(self) -> smtplib.SMTP:
        try:
            smtp = smtplib.SMTP(self.host, self.port, timeout=self.timeout)
            if self.use_tls:
                smtp.starttls()
            smtp.login(self.from_email, self.password)
            return smtp
        except Exception as error:
            print(error)
            
            
    def _handle_error(self, error: Exception):
        print(f"[SMTP Error] {error}")

    def get_provider_name(self) -> str:
        return "SMTPProvider"


from setuptools import setup, find_packages

VERSION = '0.1.13'
DESCRIPTION = 'Email manager'
LONG_DESCRIPTION = 'Email manager'

# Configurando
setup(
        name="email-manager", 
        version=VERSION,
        packages=find_packages(),
        author="Carlos Pacheco",
        license='MIT',
        author_email="carlos.pacheco@kemok.io",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        url='https://github.com/Kemok-Repos/email-manager',
)


class EmailConfig:
    def __init__(
        self,
        provider_type: str,
        timeout: int = 10,
        max_retries: int = 3,
        mailgun_params: dict = None,
        smtp_params: dict = None,
        mailtrap_params: dict = None,
    ):
        self.provider_type = provider_type.lower()
        self.timeout = timeout
        self.max_retries = max_retries
        self.mailgun_params = mailgun_params or {}
        self.smtp_params = smtp_params or {}
        self.mailtrap_params = mailtrap_params or {}




from email_manager.factory.provider_factory import ProviderFactory
from email_manager.providers.exceptions import EmailSendError

class EmailNotificationManager:
    def __init__(self, config, logger=None,):
        self.logger = logger
        self.provider_type = config.provider_type.lower()
        if not config.mailtrap_params and config.provider_type == 'mailtrap': 
            print('provider_type = mailtrap and missing mailtrap_params')
            return
        
        if not config.mailgun_params or not config.smtp_params: 
            print('missing mailgun_params or smtp_params')
            return
            
        if self.provider_type == 'mailtrap':
            self.provider = ProviderFactory.create_provider(config, override="mailtrap")
            self.fallback_provider = None
        else:
            self.provider = ProviderFactory.create_provider(config, override="mailgun")
            self.fallback_provider = ProviderFactory.create_provider(config, override="smtp")

    def send_notification(self, to: str, subject: str, body: str, reply_to, bcc, attachments: list = None) -> bool:
        try:
            return self.provider.send_email(to, subject, body, reply_to, bcc, attachments)
        except EmailSendError as e:
            print(f"⚠️ Fallback [SMTP] activado por error en {e.provider_name}: {e}")
            try:
                return self.fallback_provider.send_email(to, subject, body, reply_to, bcc, attachments)
            except EmailSendError as fallback_error:
                print(f"❌ Fallback también falló: {fallback_error}")
                return False
